/*
 * File: lcd_320_240.c
 * Date: 09.10.2012
 * Denis Zheleznyakov http://ziblog.ru
 */

#include "main.h"

struct register_data
{
	uint8_t address;
	uint16_t value;
};

const struct register_data init_table[] =
{
{ 0x00, 0x0001 },
{ 0x03, 0xA8A4 },
{ 0x0C, 0x0000 },
{ 0x0D, 0x080C },
{ 0x0E, 0x2B00 },
{ 0x1E, 0x00B0 },
{ 0x01, 0x2B3F },
{ 0x02, 0x0600 },
{ 0x10, 0x0000 },
{ 0x11, 0x6070 },
{ 0x05, 0x0000 },
{ 0x06, 0x0000 },
{ 0x16, 0xEF1C },
{ 0x17, 0x0003 },
{ 0x07, 0x0133 },
{ 0x0B, 0x0000 },
{ 0x0F, 0x0000 },
{ 0x41, 0x0000 },
{ 0x42, 0x0000 },
{ 0x48, 0x0000 },
{ 0x49, 0x013F },
{ 0x4A, 0x0000 },
{ 0x4B, 0x0000 },
{ 0x44, 0xEF00 },
{ 0x45, 0x0000 },
{ 0x46, 0x013F },
{ 0x30, 0x0707 },
{ 0x31, 0x0204 },
{ 0x32, 0x0204 },
{ 0x33, 0x0502 },
{ 0x34, 0x0507 },
{ 0x35, 0x0204 },
{ 0x36, 0x0204 },
{ 0x37, 0x0502 },
{ 0x3A, 0x0302 },
{ 0x3B, 0x0302 },
{ 0x23, 0x0000 },
{ 0x24, 0x0000 },
{ 0x25, 0x8000 },
{ 0x4f, 0x0000 },
{ 0x4e, 0x0000 } };

//------------------------------------------------------------------------------
static void write(uint16_t value)
{
	PIN_BIT_BANGING(value & BIT(0), PIN_LCD_D0);
	PIN_BIT_BANGING(value & BIT(1), PIN_LCD_D1);
	PIN_BIT_BANGING(value & BIT(2), PIN_LCD_D2);
	PIN_BIT_BANGING(value & BIT(3), PIN_LCD_D3);
	PIN_BIT_BANGING(value & BIT(4), PIN_LCD_D4);
	PIN_BIT_BANGING(value & BIT(5), PIN_LCD_D5);
	PIN_BIT_BANGING(value & BIT(6), PIN_LCD_D6);
	PIN_BIT_BANGING(value & BIT(7), PIN_LCD_D7);
	PIN_BIT_BANGING(value & BIT(8), PIN_LCD_D8);
	PIN_BIT_BANGING(value & BIT(9), PIN_LCD_D9);
	PIN_BIT_BANGING(value & BIT(10), PIN_LCD_D10);
	PIN_BIT_BANGING(value & BIT(11), PIN_LCD_D11);
	PIN_BIT_BANGING(value & BIT(12), PIN_LCD_D12);
	PIN_BIT_BANGING(value & BIT(13), PIN_LCD_D13);
	PIN_BIT_BANGING(value & BIT(14), PIN_LCD_D14);
	PIN_BIT_BANGING(value & BIT(15), PIN_LCD_D15);

	PIN_ON(PIN_LCD_WR);
	PIN_OFF(PIN_LCD_WR);
}

//------------------------------------------------------------------------------
static uint16_t read(void)
{
	uint16_t value = 0;

	if (PIN_SIGNAL(PIN_LCD_IN_D0))
	{
		value |= BIT(0);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D1))
	{
		value |= BIT(1);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D2))
	{
		value |= BIT(2);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D3))
	{
		value |= BIT(3);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D4))
	{
		value |= BIT(4);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D5))
	{
		value |= BIT(5);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D6))
	{
		value |= BIT(6);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D7))
	{
		value |= BIT(7);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D8))
	{
		value |= BIT(8);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D9))
	{
		value |= BIT(9);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D10))
	{
		value |= BIT(10);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D11))
	{
		value |= BIT(11);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D12))
	{
		value |= BIT(12);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D13))
	{
		value |= BIT(13);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D14))
	{
		value |= BIT(14);
	}
	if (PIN_SIGNAL(PIN_LCD_IN_D15))
	{
		value |= BIT(15);
	}

	return value;
}

//------------------------------------------------------------------------------
static void register_write(uint8_t address, uint16_t value)
{
	PIN_ON(PIN_LCD_CS);

	// выбор адреса регистра
	PIN_OFF(PIN_LCD_RS);
	PIN_OFF(PIN_LCD_RD);
	write(address);

	// запись данных в выбранный регистр
	PIN_ON(PIN_LCD_RS);
	write(value);

	PIN_OFF(PIN_LCD_CS);
}

//------------------------------------------------------------------------------
static uint16_t register_read(uint8_t address)
{
	uint16_t value = 0;

	PIN_ON(PIN_LCD_CS);

	PIN_OFF(PIN_LCD_RS);
	PIN_OFF(PIN_LCD_RD);
	write(address);

	//
	PIN_CONFIGURATION(PIN_LCD_IN_D0);
	PIN_CONFIGURATION(PIN_LCD_IN_D1);
	PIN_CONFIGURATION(PIN_LCD_IN_D2);
	PIN_CONFIGURATION(PIN_LCD_IN_D3);
	PIN_CONFIGURATION(PIN_LCD_IN_D4);
	PIN_CONFIGURATION(PIN_LCD_IN_D5);
	PIN_CONFIGURATION(PIN_LCD_IN_D6);
	PIN_CONFIGURATION(PIN_LCD_IN_D7);
	PIN_CONFIGURATION(PIN_LCD_IN_D8);
	PIN_CONFIGURATION(PIN_LCD_IN_D9);
	PIN_CONFIGURATION(PIN_LCD_IN_D10);
	PIN_CONFIGURATION(PIN_LCD_IN_D11);
	PIN_CONFIGURATION(PIN_LCD_IN_D12);
	PIN_CONFIGURATION(PIN_LCD_IN_D13);
	PIN_CONFIGURATION(PIN_LCD_IN_D14);
	PIN_CONFIGURATION(PIN_LCD_IN_D15);

	PIN_ON(PIN_LCD_RS);
	PIN_ON(PIN_LCD_RD);
	value = read();

	PIN_OFF(PIN_LCD_RD);

	PIN_OFF(PIN_LCD_CS);

	PIN_CONFIGURATION(PIN_LCD_D0);
	PIN_CONFIGURATION(PIN_LCD_D1);
	PIN_CONFIGURATION(PIN_LCD_D2);
	PIN_CONFIGURATION(PIN_LCD_D3);
	PIN_CONFIGURATION(PIN_LCD_D4);
	PIN_CONFIGURATION(PIN_LCD_D5);
	PIN_CONFIGURATION(PIN_LCD_D6);
	PIN_CONFIGURATION(PIN_LCD_D7);
	PIN_CONFIGURATION(PIN_LCD_D8);
	PIN_CONFIGURATION(PIN_LCD_D9);
	PIN_CONFIGURATION(PIN_LCD_D10);
	PIN_CONFIGURATION(PIN_LCD_D11);
	PIN_CONFIGURATION(PIN_LCD_D12);
	PIN_CONFIGURATION(PIN_LCD_D13);
	PIN_CONFIGURATION(PIN_LCD_D14);
	PIN_CONFIGURATION(PIN_LCD_D15);

	return value;
}

//------------------------------------------------------------------------------
void lcd_320_240_fill(uint16_t color)
{
	uint32_t i;

	register_write(0x4E, 0);
	register_write(0x4F, 0);

	PIN_ON(PIN_LCD_CS);

	// выбор адреса регистра
	PIN_OFF(PIN_LCD_RS);
	PIN_OFF(PIN_LCD_RD);
	write(0x22);

	// запись данных в выбранный регистр
	PIN_ON(PIN_LCD_RS);
	write(color);
	for(i = 1; i < 320 * 240; i++)
	{
		PIN_ON(PIN_LCD_WR);
		PIN_OFF(PIN_LCD_WR);
	}

	PIN_OFF(PIN_LCD_CS);
}

//------------------------------------------------------------------------------
void lcd_320_240_init(void)
{
	uint8_t i;

	// сброс
	PIN_ON(PIN_LCD_RST);
	PIN_OFF(PIN_LCD_RST);

	// проверка модели контроллера
	if (register_read(0x00) == 0x8989)
	{
		// инициализация
		for (i = 0; i < ARRAY_LENGHT(init_table); i++)
		{
			register_write(init_table[i].address, init_table[i].value);
		}

		// вкл. подсветку
		PIN_ON(PIN_LCD_BL);
	}
}
